<?
  require_once "common.php";
  require_once "common_dem.php";
  header("Content-type: text/plain; charset=utf-8");

  $bound_p1 = "";
  $bound_p2 = "";

  $interval = 100;
  $discrete = -1;
  
  $format = "usvx";
  
  $fname = "";
  
  if(php_sapi_name() != "cli")
  {
    $bound_p1 = $_REQUEST["p1"];
    $bound_p2 = $_REQUEST["p2"];
    
    $interval = $_REQUEST["interval"];
    if($interval[0] == "=")
      $discrete = substr($interval, 1);
    
    $format = $_REQUEST["format"];
    
  }
  else
  {
    $opt = getopt("a:b:i:d:f:F:h");
    
    if(isset($opt["a"])) $bound_p1 = $opt['a'];
    if(isset($opt["b"])) $bound_p2 = $opt['b'];
    $fname = $opt["f"];
    if(isset($opt["i"]))
    {
      $interval = $opt["i"];
      if(isset($opt["d"])) fail("Nie można jednocześnie użyć opcji -d oraz -i");
    }
    
    if(isset($opt["d"]))
      $discrete = $opt["d"];
      
    if(isset($opt["F"]))
      $format = $opt["F"];

    if(isset($opt["h"]) || (!$fname))
    {
      echo "dem.php  - Convert DEM data (HGT/DEM/GeoTiff) into contour lines suitable for importing into Survex or Walls\r\n";
      echo "           (C) 2012 - 2013 Mateusz 'mteg' Golicz <mateusz.golicz@pza.org.pl>\r\n";
      echo "\r\n";
      echo "Usage: dem.php -f <input file> [-i <interval> | -d <level>] [-a \"lat1 lon1\" -b \"lat2 lon2\"] [-F <output format>]\r\n";
      echo "\r\n";
      echo "Options:\r\n";
      echo "  -f      Input DEM file name (required)\r\n";
      echo "  -a, -b  Coordinates defining a rectangle, to which crop the input DEM before processing (default: none)\r\n";
      echo "  -F      Output format code (default: usvx)\r\n";
      echo "  -i      Contour interval (default: 100m)\r\n";
      echo "  -d      Extract only a single, discrete level (conflicts with -i)\r\n";
      echo "\r\n";
      echo "Supported output formats:\r\n";
      echo "  usvx (Survex), usrv (Walls), uacsv (CSV, absolute and UTM-projected coordinates),";
      echo " lacsv (CSV, absolute original cooridnates), uicsv (CSV, incremental UTM-projected), licsv (CSV, incremental original)\r\n";
      echo "\r\n";
      echo "The output is redirected to stdout. Use standard shell redirection to capture it into file\r\n";
      echo "Please note that the output is projected in UTM. If you are planning to superimpose cave surveys on top of the ";
      echo " output, you should rotate your surveys by the local convergence angle (so that they are oriented by topographic north). ";
      echo " Otherwise, you risk north direction differences up to 3 degrees.\r\n";
      exit;
    }

  }
  
  if($discrete == -1)
  {
    if(!is_numeric($interval))
      fail("Błąd: Niewłaściwy odstęp poziomic. \r\nError: Incorrect contour interval ($interval)");
  }
  else
    if(!is_numeric($discrete))
      fail("Błąd: Niewłaściwy poziom do ekstrakcji.\r\nError: Incorrect discrete level ($discrete)");
    
  
  $b = parse_bounds($bound_p1, $bound_p2);  
  if(php_sapi_name() != "cli")
    $fname = $_FILES["file"]["tmp_name"];

  if(!file_exists($fname))
    fail("Błąd: plik wejściowy {$fname} nie istnieje.");
  
  $gpx_path = "/tmp/dem_" . ($uid = uniqid()) . ".gpx";
  $dem_path = "/tmp/dem_" . $uid . ".tiff";
  $err_path = "/tmp/dem_" . $uid . ".err";
  
  if($bound_p1)
    do_bound($b, $fname, $dem_path, $err_path);
  else
    copy($fname, $dem_path);

  $cmd = "gdal_contour -f GPX " . ($discrete == -1 ? " -i {$interval}" : " -fl {$discrete}") . " {$dem_path} {$gpx_path}";
  system("{$cmd} >{$err_path} 2>&1");
  if(!file_exists($gpx_path))
  {
    $emsg = file_get_contents($err_path);
    @unlink($err_path); @unlink($dem_path); @unlink($gpx_path);
    fail("Błąd: Błąd wyznaczania poziomic: " . $emsg);
  }
  
  if($_REQUEST["disposition"] == "attachment")
    header("Content-disposition: attachment; filename=" . substr(md5(uniqid()), 0, 8) . ".svx");

  
  write_header($format);
  $psets = array();
  
  $fh = fopen($gpx_path, "r"); $p = array(); $m = array(); $elev = 0;
  while($l = fgets($fh))
  {
    if(preg_match('/<\/rte>/', $l))
    {
      foreach($p as $k => $v) $p[$k][2] = $elev;
      $psets[$elev][] = $p;
      $elev = 0; $p = array();
    }
    else if(preg_match('/<rtept lat="([0-9.]+)" lon="([0-9.]+)">/', $l, $m))
      $p[] = array($m[1], $m[2]);
    else if(!$elev)
      if(preg_match('/<ele>([0-9.]+)<\/ele>/', $l, $m))
        $elev = $m[1];   
  }
  fclose($fh);
 
  ksort($psets, SORT_NUMERIC);

  foreach($psets as $elev => $list)
  {
    write_elev_header($format, $elev);
    foreach($list as $p)
      finish_pset($p, $format);

    write_elev_footer($format, $elev);
  }
  
  write_footer($format);
  @unlink($err_path);
  @unlink($dem_path);
  @unlink($gpx_path);

?>