<?
  require_once "common.php";
  require_once "common_dem.php";
  header("Content-type: text/plain; charset=utf-8");

  $bound_p1 = "";
  $bound_p2 = "";

  $format = "usvx";  
  $fname = "";
  
  if(php_sapi_name() != "cli")
  {
    $bound_p1 = $_REQUEST["p1"];
    $bound_p2 = $_REQUEST["p2"];
    
    $format = $_REQUEST["format"];
  }
  else
  {
    $opt = getopt("a:b:f:F:h");
    
    if(isset($opt["a"])) $bound_p1 = $opt['a'];
    if(isset($opt["b"])) $bound_p2 = $opt['b'];
    $fname = $opt["f"];

    if(isset($opt["F"]))
      $format = $opt["F"];
    
    if(isset($opt["h"]) || (!$fname))
    {
      echo "grid.php - Convert DEM data (HGT/DEM/GeoTiff) into grid suitable for importing into Survex or Walls\r\n";
      echo "           (C) 2012 - 2013 Mateusz 'mteg' Golicz <mateusz.golicz@pza.org.pl>\r\n";
      echo "\r\n";
      echo "Usage: grid.php -f <input file> [-a \"lat1 lon1\" -b \"lat2 lon2\"] [-F <output format>]\r\n";
      echo "\r\n";
      echo "Options:\r\n";
      echo "  -f      Input DEM file name (required)\r\n";
      echo "  -a, -b  Coordinates defining a rectangle, to which crop the input DEM before processing (default: none)\r\n";
      echo "  -F      Output format code (default: usvx)\r\n";
      echo "\r\n";
      echo "Supported output formats:\r\n";
      echo "  usvx (Survex), usrv (Walls), uacsv (CSV, absolute and UTM-projected coordinates),";
      echo " lacsv (CSV, absolute original cooridnates), uicsv (CSV, incremental UTM-projected), licsv (CSV, incremental original)\r\n";
      echo "\r\n";
      echo "The output is redirected to stdout. Use standard shell redirection to capture it into file\r\n";
      echo "Please note that the output is projected in UTM. If you are planning to superimpose cave surveys on top of the ";
      echo " output, you should rotate your surveys by the local convergence angle (so that they are oriented by topographic north). ";
      echo " Otherwise, you risk north direction differences up to 3 degrees.\r\n";
      exit;
    }
  }
  
  $b = parse_bounds($bound_p1, $bound_p2);  

  if(php_sapi_name() != "cli")
    $fname = $_FILES["file"]["tmp_name"];

  if(!file_exists($fname))
    fail("Błąd: plik wejściowy {$fname} nie istnieje.\r\nError: Input file does not exist.");

  $arc_path = "/tmp/grid_" . ($uid = uniqid()) . ".gpx";
  $dem_path = "/tmp/grid_" . $uid . ".tiff";
  $err_path = "/tmp/grid_" . $uid . ".err";

  if($bound_p1)
    do_bound($b, $fname, $dem_path, $err_path);
  else
    copy($fname, $dem_path);

  $cmd = "gdal_translate -of AAIGrid {$dem_path} {$arc_path}";
  system("{$cmd} >{$err_path} 2>&1");
  if(!file_exists($arc_path))
  {
    $emsg = file_get_contents($err_path);
    @unlink($err_path); @unlink($dem_path); @unlink($arc_path);
    fail("Błąd: Błąd konwersji na postać tekstową: " . $emsg . "\r\nError: Unable to convert into text format");
  }

  
  $fh = fopen($arc_path, "r");
  $p = array(); $hdr = array();
  while($l = fgets($fh))
  {
    if(preg_match('/^[a-zA-Z]+/', $l))
    {
      list($k, $v) = explode(" ", preg_replace('/[\s]+/', ' ', trim($l)));
      $hdr[$k] = $v;
    }
  }
  fclose($fh);
  
  if(!(is_numeric($hdr["xllcorner"]) && is_numeric($hdr["yllcorner"]) && is_numeric($hdr["nrows"])
       && is_numeric($hdr["ncols"]) && is_numeric($hdr["cellsize"])))
    fail("Błąd: Niewłaściwy nagłówek pliku po konwersji do postaci tekstowej\r\nError: Unexpected header after conversion into text format");

  write_header($format);

  if($_REQUEST["disposition"] == "attachment")
    header("Content-disposition: attachment; filename=" . substr(md5(uniqid()), 0, 8) . ".svx");
  
  $y = $hdr["yllcorner"] + ($hdr["nrows"] - 1) * $hdr["cellsize"];
  $fh = fopen($arc_path, "r"); $row = 0; $cols = array();
  while($l = fgets($fh))
  {
    $l = trim($l);
    if(!is_numeric($l[0])) continue;
    $a = explode(" ", $l);
    $x = $hdr["xllcorner"];
    $p = array(); 
    foreach($a as $col => $h)
    {
      $p[] = array($x, $y, $h);
      $x += $hdr["cellsize"];
      $cols[$col][$row] = $h;
    }
    finish_pset($p, $format);
    
    $y -= $hdr["cellsize"]; $row++;
  }
  
  echo "\r\n\r\n";
  
  $x = $hdr["xllcorner"];
  foreach($cols as $rows)
  {
    $p = array();
    $y = $hdr["yllcorner"] + ($hdr["nrows"] - 1) * $hdr["cellsize"];
    foreach($rows as $h)
    {
      $p[] = array($x, $y, $h);
      $y -= $hdr["cellsize"];
    }
    finish_pset($p, $format);
    $x += $hdr["cellsize"];
  }
  
  fclose($fh);
  
  write_footer($format);
  @unlink($err_path); @unlink($dem_path); @unlink($gpx_path);

?>