<?
  require_once "common.php";
  header("Content-type: text/plain; charset=utf-8");
  
  $mode = $_REQUEST["mode"];

  if(php_sapi_name() == "cli")
  {
    $facing = 0;
    $scale = 1000;
  
    $opt = getopt("s:b:h");
    
    if(isset($opt["b"])) $facing = $opt["b"];
    if(isset($opt["s"])) $scale = $opt["s"];
    
    if(isset($opt["h"]))
    {
      echo "to_projected.php - Convert extended elevation sketch from PocketTopo export into projected elevation sketch\r\n";
      echo "                   (C) 2012 - 2013 Mateusz 'mteg' Golicz <mateusz.golicz@pza.org.pl>\r\n";
      echo "\r\n";
      echo "Usage: to_projected.php [-b <bearing>] [-s <scale>]\r\n";
      echo "\r\n";
      echo "Options:\r\n";
      echo "  -b      Projection direction in degrees (default: 0 = west-east plane)\r\n";
      echo "  -s      Scale 1:... (default: 1000)\r\n";
      echo "\r\n";
      echo "The stdin is expected to be a Therion PocketTopo export (Export / Therion...)\r\n";
      echo "The program will output a SVG file on its stdout\r\n";
      echo "\r\n";
      echo "At first, the program projects the centerline onto the specified surface. Then, for each centerline shot ";
      echo "it processes a small vertical fragment of the input sketch between the two stations involved. ";
      echo "The fragment is scaled horizontally, according to the direction the shot is 'seen' from. If necessary, ";
      echo "the fragment is flipped horizontally (this happens when the viewing angle causes the shot to be observed 'from behind'). ";
      echo "The processed fragment is then placed accordingly on top of the projected centerline.\r\n";
      echo "\r\n";
      echo "Due to the complicated nature of the conversion, the resulting sketch will almost always require manual edition. ";
      echo "Special attention is needed when the centerline 'reverses' direction, as well as in places where elevations of a few side passages are ";
      echo "superimposed on each other in the source sketch. In the latter case, the converter has no means of detecting which sketch line ";
      echo "corresponds to which passage. The sketch fragments are therefore converted multiple times and stacked together. The ";
      echo "cartographer has then to use his own reasoning to select proper lines from a bunch of nonsense.\r\n";
  
      exit;
    }
  }
  else
  {
    $facing = $_REQUEST["facing"];
    $scale = $_REQUEST["scale"];
  }
  
  if(!strlen($facing)) $facing = 0;
  if(!strlen($scale)) $scale = 1000;
  
  if(!is_numeric($facing))
    fail("Błąd: Wskazano błędny azymut rzutowania ($facing)\r\nError: Wrong projection direction.");

  if(!is_numeric($scale))
    fail("Błąd: Wskazano niepoprawną skalę ($scale)\r\nError: Wrong scale.");
  
  if($scale == 0) fail("Błąd: wskazano niepoprawną skalę (0)\r\nError: Wrong scale.");

  $filelines = load_lines(1024000);

  $s_pic = ""; $s_sect = ""; $s_color = "BLACK";
  $lnum = 0;
  
  function begin_svg()
  {
    echo "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
    echo '<svg xmlns:svg="http://www.w3.org/2000/svg" xmlns="http://www.w3.org/2000/svg" version="1.1">' . "\n";
  }
  
  function end_svg()
  {
    echo "</svg>\n";
  }
  
  $shots = array();
  $stations = array();
  $lines = array();
  $line = array();
  $shotlist = array();
  
  header("Content-type: image/svg+xml; charset=utf-8");
  header("Content-disposition: attachment; filename=" . substr(md5(uniqid()), 0, 8) . ".svg");
  foreach($filelines as $l)
  {
    $lnum++;
    $l = trim($l);
    if(!$l) continue;

    list($cmd, $junk) = explode(" ", $l);
    $process_data = false;
    
    switch($cmd)
    {
      case "PLAN": case "ELEVATION":
        $s_pic = $cmd;
        break;
      case "DATA": case "PLAN": case "STATIONS":
      case "SHOTS":
        $s_sect = $cmd;
        break;
      case "POLYLINE":
        if(count($line))
          $lines[$s_color][] = $line;

        $s_sect = $cmd;
        $s_color = $junk;
        $line = array();
        break;
      default:
        $process_data = true;
        break;
    }
    
    if($process_data)
    {
      switch($s_sect)
      {
        case "DATA":
          list($from, $to, $compass, $clino, $tape) = explode("\t", $l);
          if(!(is_numeric($compass) && is_numeric($clino) && is_numeric($tape)))
            fail("Incorrect DATA in line {$lnum}");
          
          if($to)
          {
            $backcompass = $compass + 180;
            if($backcompass > 360) $backcompass -= 360;
            $shots[$from][$to] = array($compass, $clino, $tape);
            $shots[$to][$from] = array($backcompass, -$clino, $tape);
            $shotlist[] = array($from, $to);
          }
          break;
        case "STATIONS":
          if($s_pic == "ELEVATION")
          {
            list($x, $y, $station) = explode("\t", $l);
            if(!isset($stations[$station]))
              $stations[$station] = array($x, $y);
          }     
          break;
        case "POLYLINE":
          if($s_pic == "ELEVATION")
          {
            list($x, $y) = explode("\t", $l);
            $line[] = array($x, $y);
          }
          break;
      }
    }
  }
  if(count($line)) $lines[$s_color][] = $line;
  
  /* Solve */
  $fix[key($stations)] = array(0, 0);
  $itercount = 0;
  
  begin_svg();
  while(count($fix) < count($stations))
  {
    $itercount ++;
    if($itercount > 100) fail("Error: Cannot solve station graph after {$itercount} iterations");
    /* For each fixed station */
    foreach($fix as $s => $p)
    {
      list($x, $y) = $p;

      /* Try to reach everything from this station */
      foreach($shots[$s] as $t => $si)
      {
        if(isset($fix[$t]))
        {
          /* Destination station already fixed, shot not useful */
          unset($shots[$s][$t]);
          continue;
        }
        
        /* Can fix a new station! */
        list($compass, $clino, $tape) = $si;
        $dP = cos($clino / 180 * pi()) * $tape;
        
        $dY = sin($clino / 180 * pi()) * $tape;
        $dX = sin(($compass - $facing) / 180 * pi()) * $dP; 
        
        $fix[$t] = array($x + $dX, $y + $dY);
        unset($shots[$s][$t]);
      }
    }
  }

  foreach($shotlist as $ent)
  {
    list($from, $to) = $ent;
    
    /* Source X range */
    $src_x1 = $stations[$from][0];
    $src_x2 = $stations[$to][0];

    /* Target X range */
    $dst_x1 = $fix[$from][0];
    $dst_x2 = $fix[$to][0];
    
    $dir = ($src_x2 - $src_x1) * ($dst_x2 - $dst_x1);
    $dir = $dir > 0 ? 1 : -1;
    
    $src_xs = min($src_x1, $src_x2);
    $src_xe = max($src_x1, $src_x2);
    $dst_xs = min($dst_x1, $dst_x2);
    $dst_xe = max($dst_x1, $dst_x2);
    
    echo "  <g fill='none' stroke-width='0.1mm'>\n";
//    echo "shot $from, $to: must map {$src_xs} - {$src_xe} to {$dst_xs} - {$dst_xe} dir {$dir}\n";
//  draw show
    echo sprintf("    <circle cx='%.2fmm' cy='%.2fmm' r='.5mm' fill='cyan'/>\n", $fix[$from][0], -$stations[$from][1]);
    echo sprintf("    <circle cx='%.2fmm' cy='%.2fmm' r='.5mm' fill='cyan'/>\n", $fix[$to][0], -$stations[$to][1]);
    echo sprintf("    <line x1='%.2fmm' y1='%.2fmm' x2='%.2fmm' y2='%.2fmm' stroke='cyan' stroke-width='0.2mm'/>\n",
        $fix[$from][0], -$stations[$from][1], $fix[$to][0], -$stations[$to][1]);

    echo sprintf("    <text x='%.2fmm' y='%.2fmm' font-family='verdana' font-size='.25mm' fill='black'>%s</text>\n", $fix[$from][0] - 0.3, -$stations[$from][1] - 0.0, $from);
    echo sprintf("    <text x='%.2fmm' y='%.2fmm' font-family='verdana' font-size='.25mm' fill='black'>%s</text>\n", $fix[$to][0] - 0.3, -$stations[$to][1] - 0.0, $to);


    
    foreach($lines as $color => $linelist)
    /* For each color */
    {
      echo "  <g stroke='" . strtolower($color) . "'>\n";
      foreach($linelist as $line)
      /* For each line in this color */
      {
        $x1 = false; $y1 = false;
        foreach($line as $p)
        {
          list($x2, $y2) = $p;
          if(!($x1 === false))
          {
            /* For each line segment (x1, y1) - (x2, y2) */

            /* Normalize line segment (ensure that always x1 < x2) */
            if($x1 > $x2)
            {
              /* Swap ends */
              $x2 = $x1; $y2 = $y1;
              list($x1, $y1) = $p;
            }
            
            /* Sort out lines not intersecting current slice */
            if($x1 > $src_xe) continue;
            if($x2 < $src_xs) continue;
           
//            echo "clipping line ($x1, $y1) - ($x2, $y2) to ($src_xs, $src_xe)\n";
           
            /* Clip line to this slice */
            if($x1 < $src_xs)
            {
              $y1 = $y1 + ($y2 - $y1) / ($x2 - $x1) * ($src_xs - $x1);
              $x1 = $src_xs;
            } 

            if($x2 > $src_xe)
            {
              $y2 = $y2 - ($y2 - $y1) / ($x2 - $x1) * ($x2 - $src_xe);
              $x2 = $src_xe;
            } 
//            echo "  clipped to ($x1, $y1) - ($x2, $y2)\n";
            
            /* Transform line segment to new coordinate system */
            $x1 -= $src_xs;
            $x1 /= ($src_xe - $src_xs);
            $x1 *= ($dst_xe - $dst_xs);
            
            if($dir > 0)
              $x1 += $dst_xs;
            else
              $x1 = $dst_xe - $x1;

            $x2 -= $src_xs;
            $x2 /= ($src_xe - $src_xs);
            $x2 *= ($dst_xe - $dst_xs);

            if($dir > 0)
              $x2 += $dst_xs;
            else
              $x2 = $dst_xe - $x2;
            
            $x1 = round($x1, 2);
            $x2 = round($x2, 2);
            $y1 = round($y1, 2);
            $y2 = round($y2, 2);
            
            $y1 = -$y1;
            $y2 = -$y2;
            
            if($dir < 0)
            {
            }
            
 //           echo "LINE $x1, $y1 - $x2, $y1\n";
            echo "    <line x1='{$x1}mm' y1='{$y1}mm' x2='{$x2}mm' y2='{$y2}mm' />\n";
            
          }
          list($x1, $y1) = $p;
        }
      }
      echo "  </g>\n";
    }
    echo "  </g>\n";
  }
  end_svg();
  exit;
  
  print_r($fix);
  print_r($stations);
  print_r($shots);
  print_r($lines);
  
?>