<?
  require_once "common.php";
  header("Content-type: text/plain; charset=utf-8");

  $lat = 1;
  $lon = 2;  
  $dir = 0;
  $limit = 8;

  if(php_sapi_name() != "cli")
  {
    $separator = ";";
    $cal_separator = ";";
    
    $separator = $_REQUEST["separator"];
    $cal_separator = $_REQUEST["cal_separator"];
    $decimals = $_REQUEST["decimals"];
    $lat = $_REQUEST["lat"];
    $lon = $_REQUEST["lon"];
    $dir = $_REQUEST["dir"];
//    $limit = $_REQUEST["limit"];
    
  }
  else
  {
    $separator = "\t"; $cal_separator = "\t";
    $opt = getopt("s:S:x:y:rd:hc:l:e:");
    
    if(isset($opt["x"])) $lat = $opt['x'];
    if(isset($opt["y"])) $lon = $opt['y'];
    if(isset($opt["s"])) $separator = $opt['s'];
    if(isset($opt["d"])) $decimals = $opt['d'];
    if(isset($opt["S"])) $cal_separator = $opt['S'];
    if(isset($opt["r"])) $dir = 1;    
    if(isset($opt["l"])) $limit = $opt['l'];
    if(isset($opt["e"])) $caleval = $opt['e'];

    if(isset($opt["h"]) || !$opt["c"])
    {
      echo "transform.php - Determine a transformation between coordinate systems and transform points between the systems\r\n";
      echo "                (C) 2012 - 2013 Mateusz 'mteg' Golicz <mateusz.golicz@pza.org.pl>\r\n";
      echo "\r\n";
      echo "Usage: transform.php -c <calibration file> [-d <separator>] [-x <column>] [-y <column>] [-S <separator>] [-s <separator>] [-r] [-l <limit>] [-e <seconds>]\r\n";
      echo "\r\n";
      echo "Options:\r\n";
      echo "  -c      Calibration file (CSV)\r\n";
      echo "  -S      Column separator in the calibration file (default: <Tab>)\r\n";
      echo "  -s      Column separator in input (default: <Tab>)\r\n";
      echo "  -x, -y  Numbers of columns containing the X and Y coordinates in input and output (default: 1, 2)\r\n";
      echo "  -l      Use only first few calibration points (default: 8)\r\n";
      echo "  -r      Perform inverse transformation (from 2 into 1)\r\n";
      echo "  -e      Random calibration improvement mode (no data will be converted)\r\n";
      echo "  -d      Decimals separator (only , and . allowed)\r\n";
      echo "\r\n";
      echo "The input calibration file is supposed to contain coordinates of calibration points (one point in row) in both systems (1 and 2). At least three points are required. ";
      echo "The columns should contain: X coordinate in system 1 (column 1), Y in system 1, X in system 2, Y in system 2 (column 4).\r\n";
      echo "\r\n";
      echo "The program finds a linear map between two coordinate systems, looking for a matrix A and vector B, so that Vn = A x Un + B ";
      echo "is satisfied for each calibration point (Vn, Un - coordinates of n-th calibration point in system 1 and 2). If more than ";
      echo "3 points are given, the program looks for matrix A and vectors B, K4, K5, .. so that for each calibration point, Vn = A x Un + B + K4 * dist(V4, Un) + K5 * dist(V5, Un) + ...\r\n";
      echo "\r\n";
      echo "After finding the linear map, the program reads point coordinates from stdin. The input is assumed to be in a CSV-like form. ";
      echo "The coordinates are transformed using the determined map and output on the stdout. Columns not holding coordinates are left untouched ";
      echo "Lines not containing numeric values in the ";
      echo "coordinate columns are ignored and copied to the output as they are. \r\n";
      echo "\r\n";
      echo "Please note, that the more calibration points, the longer solving the equation takes. A reasonable maximum is 7 - 8 points. \r\n";
      exit;
    }



    if(!($calfile = $opt["c"]))
      fail("Nie wskazano pliku kalibracyjnego!\r\nError: Calibration file not specified!");
    
    if(!file_exists($calfile))
      fail("Plik kalibracyjny $calfile nie istnieje!\r\nError: Calibration file does not exist.");
  }
  
  if(!is_numeric($limit)) fail("Błąd: nieprawidłowy limit ilości punktów kalibracyjnych do użycia!\r\nError: incorrect limit of calibration points to use");
  if(!is_numeric($lat)) fail("Błąd: wskazano błędny numer kolumny z szerokością/northing: $lat\r\nError: incorrect column number for latitude/northing.");
  if(!is_numeric($lon)) fail("Błąd: wskazano błędny numer kolumny z długością/easting: $lon\r\nError: incorrect column number for longitude/easting");
  
  if(php_sapi_name() != "cli")
    if($limit > 8)
      fail("Zbyt wysoki limit ilości punktów kalibracyjnych\r\nError: Too large calibration points limit");
  
  $lat -= 1; $lon -= 1;
  
  if($separator == "t") $separator = "\t";
  if($cal_separator == "t") $cal_separator = "\t";

  $filelines = load_lines(1024000);
  $callines = load_lines(262144, "cal", "caldata", $calfile);


  // Calibrate!
  
  $cal = ParseCal_Cal($callines, $cal_separator, $dir); 
  if(count($cal) < 3) fail("Błąd: Zbyt mało punktów kalibracyjnych\r\nError: Too little calibration points");
  
  function CalRstr($cal, $rstr)
  {
      $cal_a = array();
      foreach(explode(",", $rstr) as $n)
        $cal_a[] = $cal[$n];
      $mLat = FindCalibrationMatrixA($cal_a, "lat", "x", "y");  
      $mLon = FindCalibrationMatrixA($cal_a, "lon", "x", "y"); 
      return(array($cal_a, $mLat, $mLon));
  }
  
  function CalGetErrors($cal, $cal_a, $mLat, $mLon, $verbose = false)
  {
    $sumdelta = 0;
    foreach($cal as $k => $e)
    {
      $lat = $e["lat"]; $lon = $e["lon"]; $x = $e["x"]; $y = $e["y"];

      $lat_a = DeprojectPoint($x, $y, $mLat, $cal_a, "x", "y");
      $lon_a = DeprojectPoint($x, $y, $mLon, $cal_a, "x", "y");      
      $error = sqrt(pow($eX = abs($lat_a - $lat), 2) + pow($eY = abs($lon_a - $lon), 2));

      if($verbose) echo sprintf("# %2d: X2=%.6f Y2=%.6f  X1=%.6f   Y1=%.6f    nX2=%.6f  nY2=%.6f   eX=%.6f  eY=%.6f  delta=%.6f\r\n", $k, $lat, $lon, $x, $y, $lat_a, $lon_a, $eX, $eY, $error);

      $sumdelta += $error;
    }
    return $sumdelta;
  }

  if(is_numeric($caleval))
  {
    if(count($cal) <= $limit) fail("Błąd: z kalibracji nie ma czego odrzucać!\r\nError: Nothing to drop from the calibration data.");
    $t_0 = microtime(true); $tests = array(); $min_error = 0; $min_rstr = false;
    while((microtime(true) - $t_0) < $caleval)
    {
      $iterations++;
      $cal_b = $cal; $rstr = array();

      for($i = 0; $i<$limit; $i++)
      {
        $n = array_rand($cal_b);
        $rstr[] = $n;
        unset($cal_b[$n]);
      }
      
      sort($rstr, SORT_NUMERIC);
      $rstr = implode(",", $rstr);
      if(isset($tests[$rstr])) continue;
      
      list($cal_a, $mLat, $mLon) = CalRstr($cal, $rstr);
    
      $error = CalGetErrors($cal, $cal_a, $mLat, $mLon);
      $tests[$rstr] = $error; 
      if($error < $min_error || !$min_rstr)
      {
        $min_rstr = $rstr;
        $min_error = $error;
      }
    }
    
    echo "# Najlepszy wynik przeszukania " . count($tests) . " zestawów punktów kalibracyjnych po " . $limit . " punktów każdy: \r\n";
    $rstr = explode(",", $min_rstr);
    
    foreach($cal as $k => $v)
    {
      if(!in_array($k, $rstr)) echo "# ";
      echo $v["x"] . $cal_separator . $v["y"] . $cal_separator . $v["lat"] . $cal_separator . $v["lon"] . $cal_separator . "\r\n";
    }

    list($cal_a, $mLat, $mLon) = CalRstr($cal, $min_rstr);
    echo "# Łączny wskaźnik dobroci kalibracji: " . CalGetErrors($cal, $cal_a, $mLat, $mLon, true)/(count($cal) - $limit) . "\r\n";
    exit;
  }
  
  $cal = array_slice($cal, 0, $limit);
  $mLat = FindCalibrationMatrixA($cal, "lat", "x", "y");  
  $mLon = FindCalibrationMatrixA($cal, "lon", "x", "y"); 
  
  foreach($filelines as $l)
  {
    $l = trim($l, "\r\n");
    if(!strlen($l)) continue;

    $a = explode($separator, $l);
    $x = trim(strtr($a[$lat], array('"' => '', ',' => '.')));
    $y = trim(strtr($a[$lon], array('"' => '', ',' => '.')));
    
    if(is_numeric($x) && is_numeric($y))
    {
      $lat_a = DeprojectPoint($x, $y, $mLat, $cal, "x", "y");
      $lon_a = DeprojectPoint($x, $y, $mLon, $cal, "x", "y");      
      $a[$lat] = strtr($lat_a, ($decimals == ",") ? array("." => ",") : array());
      $a[$lon] = strtr($lon_a, ($decimals == ",") ? array("." => ",") : array());
      echo implode($separator, $a) . "\r\n";
    }
    else
      echo $l . "\r\n";
  }

?>