<?
  require_once "common.php";
  header("Content-type: text/plain; charset=utf-8");
  


  if(php_sapi_name() == "cli")
  {
    $separator = "\t";
    $dir = 0;
    
    $opt = getopt("z:s:x:y:d:rh");
    
    if(isset($opt["x"])) $lon = $opt['x'];
    if(isset($opt["y"])) $lat = $opt['y'];
    if(isset($opt["s"])) $separator = $opt['s'];
    if(isset($opt["z"])) $zone = $opt['z'];
    if(isset($opt["r"])) $dir = 1;    
    if(isset($opt["d"])) $decimals = $opt['d'];

    if(isset($opt["h"]))
    {
      echo "utm.php - Project or deproject coordinates between Lat/Lon WGS84 and UTM\r\n";
      echo "                (C) 2012 - 2013 Mateusz 'mteg' Golicz <mateusz.golicz@pza.org.pl>\r\n";
      echo "\r\n";
      echo "Usage: utm.php [-d <separator>] [-x <column>] [-y <column>] [-s <separator>] [-r] [-z <zone>]\r\n";
      echo "\r\n";
      echo "Options:\r\n";
      echo "  -s      Column separator in input (default: <Tab>)\r\n";
      echo "  -x, -y  Numbers of columns containing the X (Lon or Easting) and Y (Lat or Northing) coordinates in input and output (default: 1, 2)\r\n";
      echo "  -r      Perform inverse projection (from UTM to Lat/Lon) - default is Lat/Lon to UTM\r\n";
      echo "  -d      Decimals separator (only , and . allowed)\r\n";
      echo "  -z      UTM zone number (only digits). Optional in case of forward projection. Required with -r.\r\n";
      echo "\r\n";
      echo "The program reads point coordinates from stdin. The input is assumed to be in a CSV-like form. ";
      echo "The coordinates are transformed and output on the stdout in the same columns as they were read. \r\n";
      echo "Columns not holding coordinates are left untouched ";
      echo "Lines not containing numeric values in the ";
      echo "coordinate columns are ignored and copied to the output as they are. \r\n";
      exit;
    }

  }
  else
  {
    $zone = $_REQUEST["zone"];
    $separator = $_REQUEST["separator"];
    $lat = $_REQUEST["lat"];
    $lon = $_REQUEST["lon"];
    $decimals = $_REQUEST["decimals"];
    $dir = $_REQUEST["dir"];

    if(!$separator) 
      $separator = ";";
  }
  
  if($zone && !is_numeric($zone))
    fail("Błąd: Wskazano błędny numer strefy UTM ($zone)\r\nError: Wrong UTM zone number.");
  
  if($dir)
    if(!strlen($zone))
      fail("Błąd: Nie wskazano numeru strefy UTM\r\nError: No UTM zone specified");
  
  if(!strlen($lat)) $lat = 1;
  if(!strlen($lon)) $lon = 2;
  
  if(!is_numeric($lat)) fail("Błąd: wskazano błędny numer kolumny z szerokością/northing: $lat\r\nError: incorrect column number for latitude/northing.");
  if(!is_numeric($lon)) fail("Błąd: wskazano błędny numer kolumny z długością/easting: $lon\r\nError: incorrect column number for longitude/easting");
  
  $lat -= 1; $lon -= 1;
  
  
  if($separator == "t") $separator = "\t";

  $filelines = load_lines(1024000);
  
  
  $p = array();
  foreach($filelines as $l)
  {
    $l = trim($l, "\r\n");

    $a = explode($separator, $l);
    $p_lat = trim(strtr($a[$lat], array('"' => '')));
    $p_lon = trim(strtr($a[$lon], array('"' => '')));
    
    if($decimals == ",")
    {
      $p_lat = strtr($p_lat, array("," => "."));
      $p_lon = strtr($p_lon, array("," => "."));
    }
    
    if($dir)
    {
      if(is_numeric($p_lat) && is_numeric($p_lon))
      {
        list($p_lat, $p_lon) = utm2ll($p_lon, $p_lat, $zone);
        $a[$lat] = round($p_lat, 6);
        $a[$lon] = round($p_lon, 6);
        $valid = true;
      }
    }
    else
    {
      if(is_array($c = get_coordinates($p_lat . "\t" . $p_lon)))
      {        
        if(!$zone)
          $zone = select_utm_zone($c["lat"], $c["lon"]);
        list($x, $y) = ll2utm($c["lat"], $c["lon"], $zone);
        $a[$lat] = round($y, 2);
        $a[$lon] = round($x, 2);
        $valid = true;
      }
    }
    if($valid)
    {
      if($decimals == ",")
      {
        foreach(array($lat, $lon) as $k)
          $a[$k] = strtr($a[$k], array("." => ","));
      }
      echo implode($separator, $a) . "\r\n";
    }
    else
      echo $l . "\r\n";
  }

?>